"""Gateway connecting to ATAG thermostat."""
import logging
import aiohttp

from .const import (DEFAULT_SENSOR_SET, RETRIEVE_PATH, UPDATE_PATH, PAIR_PATH, PAIR_REPLY,
                    UPDATE_REPLY, RETRIEVE_REPLY, DETAILS, ACC_STATUS, REPORT, SENSOR_TYPES,
                    BOILER_STATUS, BOILER_STATES, CONTROL, INT_MODES, ATTR_OPERATION_MODE,
                    ATTR_OPERATION_MODE_INT, ATTR_REPORT_TIME)
from .helpers import HostData, HttpConnector, get_time_from_stamp
from .errors import ResponseError, RequestError

_LOGGER = logging.getLogger(__name__)


class AtagDataStore:
    """Central data store entity."""

    def __init__(
            self, session=None, host=None, port=None,
            mail=None, interface=None, sensors=None):
        try:
            self.host_data = HostData(
                host=host, port=port, interface=interface, mail=mail)
        except RequestError:
            _LOGGER.error("Incorrect host data provided, check config!")
        if session is None:
            session = aiohttp.ClientSession()
        if type(session).__name__ == 'ClientSession':
            self._connector = HttpConnector(self.host_data, session)
        else:
            _LOGGER.error("Not a valid session: %s", type(session).__name__)

        self.data = {}
        if sensors is None:
            self.sensors = DEFAULT_SENSOR_SET
        else:
            self.sensors = sensors
        self.sensordata = {}
        self.paired = False

    async def async_update(self):
        """Read data from thermostat."""
        if not self.paired:
            _LOGGER.debug("Atag not paired yet - attempting..")
            if not await self.async_check_pair_status():
                _LOGGER.error("Pairing failed - please confirm pairing on device")
                return
        try:
            json_data = await self._connector.atag_put(
                data=self.host_data.retrieve_msg, path=RETRIEVE_PATH)
        except ResponseError as err:
            _LOGGER.warning("Atag failed to update:\n %s", err)
            return False
        sensordata = self.store_sensor_data(json_data)
        if sensordata:
            self.sensordata = sensordata

    async def async_set_atag(self, _target_mode=None, _target_temp=None):
        """set mode and/or temperature."""
        json_payload = self.host_data.get_update_msg(
            _target_mode, _target_temp)
        _LOGGER.debug(
            "Updating: Mode:[%s], Temp:[%s]", _target_mode, _target_temp)
        try:
            json_data = await self._connector.atag_put(data=json_payload, path=UPDATE_PATH)
            _LOGGER.debug("Update reply: %s", json_data)
            if not json_data[UPDATE_REPLY][ACC_STATUS] == 2:
                raise ResponseError("Invalid update reply received: {}".format(json_data))
        except (ResponseError, KeyError) as err:
            _LOGGER.error("Failed to set Atag: %s", err)
            return False

    def store_sensor_data(self, json_response):
        """Store relevant sensor data directly from json."""
        result = {}
        try:
            status = json_response[RETRIEVE_REPLY][REPORT]
            for sensor in self.sensors:
                datafield = SENSOR_TYPES[sensor][3]
                if sensor == BOILER_STATUS:
                    worker = int(status[sensor])
                    result[sensor] = BOILER_STATES[worker & 14]
                elif DETAILS in status and datafield in status[DETAILS]:
                    result[sensor] = float(status[DETAILS][datafield])
                elif datafield in status:
                    result[sensor] = float(status[datafield])
                else:
                    raise ResponseError("Atag sensor {} type error: {}".format(sensor, datafield))
                mode_int = json_response[RETRIEVE_REPLY][CONTROL][ATTR_OPERATION_MODE_INT]
                result[ATTR_OPERATION_MODE] = INT_MODES[mode_int]
                result[ATTR_REPORT_TIME] = get_time_from_stamp(
                    status[ATTR_REPORT_TIME])
        except (ResponseError, KeyError) as err:
            _LOGGER.error('Atag sensor failed to update: %s', err)
            return False
        return result

    async def async_check_pair_status(self):
        """Confirm we are authorized."""
        if self.paired:
            return True
        try:
            json_data = await self._connector.atag_put(data=self.host_data.pair_msg, path=PAIR_PATH)
            status = json_data[PAIR_REPLY][ACC_STATUS]
            _LOGGER.debug("AtagDataStore pairing\n%s\n%s\n%s",
                          self.host_data.pair_msg, json_data, status)
        except ResponseError as err:
            _LOGGER.error("Pairing failed\n%s\n%s",
                          self.host_data.pair_msg, err)
            return False
        if status == 2:
            self.paired = True
            _LOGGER.debug("AtagDataStore paired")
            return True
        if status == 1:
            _LOGGER.warning("Waiting for pairing confirmation")
        elif status == 3:
            _LOGGER.warning("Waiting for pairing confirmation")
        elif status == 0:
            _LOGGER.warning("No status returned from ATAG One")
        _LOGGER.warning("Atag not paired!\n%s", json_data)
        return False

    async def async_close(self):
        """Close the connection"""
        await self._connector.async_close()



"""Constants for ATAG API."""

DOMAIN = 'atag'
ATAG_HANDLE = 'atag_data'
DATA_LISTENER = 'atag_listener'
REQUEST_INFO = 64 + 8 + 1
SIGNAL_UPDATE_ATAG = 'atag_update'
CONF_INTERFACE = 'interface'

STATE_HEAT = 'heat'
STATE_ECO = 'eco'
STATE_AUTO = 'auto'
STATE_MANUAL = 'manual'
ATTR_OPERATION_MODE = 'operation_mode'

DEFAULT_TIMEOUT = 15
DEFAULT_PORT = 10000
DEFAULT_SCAN_INTERVAL = 120
DEFAULT_INTERFACE = 'eth0'
DEFAULT_SENSOR_SET = {
    'current_temp',
    'temperature',
    'boiler_status',
    'flame_level'
}
HTTP_HEADER = {
    'Content-type': 'applicaton/x-www-form-urlencoded;charset=UTF-8',
    'Connection': 'Close',
    'User-Agent': 'Mozilla/5.0 (compatible; AtagOneAPI/x; http://atag.one/)'
}

# ATTR_CURRENT_HUMIDITY = 'current_humidity'
ATTR_CURRENT_TEMPERATURE = 'current_temperature'

# ATTR_HUMIDITY = 'humidity'
# ATTR_MAX_HUMIDITY = 'max_humidity'
ATTR_MAX_TEMP = 'max_temp'
# ATTR_MIN_HUMIDITY = 'min_humidity'
ATTR_MIN_TEMP = 'min_temp'
ATTR_OPERATION_LIST = 'operation_list'
ATTR_OPERATION_MODE = 'operation_mode'
ATTR_OPERATION_MODE_INT = 'ch_mode'
ATTR_TEMPERATURE_SET = 'shown_set_temp'
ATTR_TEMPERATURE = 'temperature'
ATTR_REPORT_TIME = 'report_time'
# ATTR_TARGET_TEMP_HIGH = 'target_temp_high'
# ATTR_TARGET_TEMP_LOW = 'target_temp_low'
# ATTR_TARGET_TEMP_STEP = 'target_temp_step'
BOILER_STATUS = 'boiler_status'

SENSOR_TYPES = {
    ATTR_CURRENT_TEMPERATURE: ['Current Temperature', '°C', 'mdi:thermometer', 'room_temp'],
    'outside_temp': ['Outside Temp', '°C', 'mdi:thermometer', 'outside_temp'],
    'outside_temp_avg': ['Average Outside Temperature', '°C', 'mdi:thermometer',
                         'tout_avg'],
    'pcb_temp': ['PCB Temperature', '°C', 'mdi:thermometer', 'pcb_temp'],
    ATTR_TEMPERATURE: ['Target Temperature', '°C', 'mdi:thermometer', ATTR_TEMPERATURE_SET],
    'ch_water_pressure': ['Central Heating Pressure', 'Bar', 'mdi:gauge',
                          'ch_water_pres'],
    'ch_water_temp': ['Central Heating Water Temperature', '°C', 'mdi:thermometer',
                      'ch_water_temp'],
    'ch_return_temp': ['Central Heating Return Temperature', '°C', 'mdi:thermometer',
                       'ch_return_temp'],
    'dhw_water_temp': ['Hot Water Temperature', '°C', 'mdi:thermometer',
                       'dhw_water_temp'],
    'dhw_water_pres': ['Hot Water Pressure', 'Bar', 'mdi:gauge',
                       'dhw_water_pres'],
    BOILER_STATUS: ['Boiler Status', '', 'mdi:flash', 'boiler_status'],
    'boiler_config': ['Boiler Config', '', 'mdi:flash', 'boiler_config'],
    'water_pressure': ['Boiler Pressure', 'Bar', 'mdi:gauge',
                       'water_pressure'],
    'burning_hours': ['Burning Hours', 'h', 'mdi:fire', 'burning_hours'],
    'voltage': ['Voltage', 'V', 'mdi:flash', 'voltage'],
    'current': ['Current', 'mA', 'mdi:flash-auto', 'current'],
    'flame_level': ['Flame', '%', 'mdi:fire', 'rel_mod_level'],
}

BOILER_STATES = {
    14: 'Heating CV & Water',
    12: 'Heating Water',
    10: 'Heating CV',
    8: 'Heating Boiler',
    4: 'Water active',
    2: 'CV active',
    0: 'Idle'
}

# fix for Google Assistant integration - use HEAT instead of MANUAL
MODES = {STATE_HEAT: 1, STATE_AUTO: 2}
INT_MODES = {v: k for k, v in MODES.items()}

UPDATE_MODE = 'update_mode'
UPDATE_TEMP = 'update_temp'
PAIR_PATH = 'pair'
UPDATE_PATH = 'update'
RETRIEVE_PATH = 'retrieve'
UPDATE_REPLY = 'update_reply'
RETRIEVE_REPLY = 'retrieve_reply'
PAIR_REPLY = 'pair_reply'
REPORT = 'report'
CONTROL = 'control'
DETAILS = 'details'
ACC_STATUS = 'acc_status'

DEFAULT_NAME = 'Atag One Thermostat'
DEFAULT_MIN_TEMP = 12
DEFAULT_MAX_TEMP = 21

SENSOR_PREFIX = 'Atag '
